package me.unurled.sacredrealms.sr.commands.admin;

import static me.unurled.sacredrealms.sr.utils.Component.comp;
import static me.unurled.sacredrealms.sr.utils.Logger.error;

import java.util.Arrays;
import java.util.List;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.item.Item;
import me.unurled.sacredrealms.sr.components.item.ItemManager;
import me.unurled.sacredrealms.sr.components.item.ItemType;
import me.unurled.sacredrealms.sr.components.item.Rarity;
import me.unurled.sacredrealms.sr.components.item.abilities.Ability;
import me.unurled.sacredrealms.sr.components.item.enchantments.Enchantment;
import me.unurled.sacredrealms.sr.managers.Manager;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class ItemCommand implements TabExecutor {

  public static final String DELETE = "delete";
  public static final String DOES_NOT_EXIST = "<red>An item with that ID does not exist.";
  public static final String MODIFY = "modify";
  public static final String MATERIAL = "material";
  public static final String ENCHANTMENTS = "enchantments";
  public static final String ATTRIBUTES = "attributes";
  public static final String ABILITIES = "abilities";
  public static final String CUSTOMMODELDATA = "custommodeldata";
  public static final String RARITY = "rarity";
  public static final String REMOVE = "remove";
  public static final String USAGE =
      "Usage: /item modify <id> <name|material|enchantments|attributes|abilities|custommodeldata|rarity|type|description> [value]";

  private static void modify(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p) {
    if (args.length < 2) {
      p.sendMessage(USAGE);
      return;
    }

    ItemManager im = Manager.getInstance(ItemManager.class);
    if (!im.getItemIDs().contains(args[1])) {
      sender.sendMessage(comp(DOES_NOT_EXIST));
      return;
    }

    Item item = im.getItem(args[1]);

    if (args.length == 2) {
      p.sendMessage(USAGE);
      return;
    }

    boolean modified = false;

    switch (args[2].toLowerCase()) {
      case "name" -> modified = modifyName(sender, args, p, item);
      case MATERIAL -> modified = modifyMaterial(sender, args, p, item);
      case ENCHANTMENTS -> modified = modifyEnchantments(args, p, item);
      case ATTRIBUTES -> modified = modifyAttributes(sender, args, p, item);
      case ABILITIES -> modified = modifyAbilities(sender, args, p, item);
      case CUSTOMMODELDATA -> modified = modifyModelData(sender, args, p, item);
      case RARITY -> modified = modifyRarity(sender, args, p, item);
      case "type" -> modified = modifyType(sender, args, p, item);
      case "description" -> modified = modifyDescription(sender, args, p, item);
      default -> {
        p.sendMessage(USAGE);
        return;
      }
    }

    // save the item
    if (item != null && modified) {
      im.saveItem(item);
    }
  }

  private static boolean modifyDescription(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> description <value>");
      return false;
    }
    String description = String.join(" ", Arrays.copyOfRange(args, 3, args.length));
    item.setDescription(description);
    sender.sendMessage(comp("<green>Description set."));
    updateItem(p, item);
    return true;
  }

  private static boolean modifyType(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> type <value>");
      return false;
    }
    ItemType type;
    try {
      type = ItemType.valueOf(args[3]);
    } catch (Exception e) {
      sender.sendMessage(comp("<red>Type not found."));
      return false;
    }
    item.setType(type);
    sender.sendMessage(comp("<green>Type set."));
    updateItem(p, item);
    return true;
  }

  private static boolean modifyRarity(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> rarity <value>");
      return false;
    }
    Rarity rarity;
    try {
      rarity = Rarity.valueOf(args[3]);
    } catch (Exception e) {
      sender.sendMessage(comp("<red>Rarity not found."));
      return false;
    }
    item.setRarity(rarity);
    sender.sendMessage(comp("<green>Rarity set."));
    updateItem(p, item);
    return true;
  }

  private static boolean modifyModelData(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> custommodeldata <value>");
      return false;
    }
    int customModelData;
    try {
      customModelData = Integer.parseInt(args[3]);
    } catch (NumberFormatException e) {
      sender.sendMessage(comp("<red>Invalid value."));
      return false;
    }
    item.setCustomModelData(customModelData);
    sender.sendMessage(comp("<green>Custom model data set."));
    updateItem(p, item);
    return true;
  }

  private static boolean modifyAbilities(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 5) {
      p.sendMessage("Usage: /item modify <id> abilities add <ability>");
      return false;
    }
    if (args[3].equalsIgnoreCase("add")) {
      p.sendMessage("Abilities not working for now.");
      item.addAbility(Ability.getAbility(args[4]));
    } else if (args[3].equalsIgnoreCase(REMOVE)) {
      sender.sendMessage("Abilities not working for now.");
      item.removeAbility(Ability.getAbility(args[4]));
    }
    updateItem(p, item);
    return true;
  }

  private static boolean modifyName(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> name <value>");
      return false;
    }
    String name = String.join(" ", Arrays.copyOfRange(args, 3, args.length));
    item.setName(name);
    sender.sendMessage(comp("<green>Item name set."));
    updateItem(p, item);
    return true;
  }

  private static boolean modifyMaterial(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> material <value>");
      return false;
    }
    Material material = Material.matchMaterial(args[3]);
    if (material == null) {
      sender.sendMessage(comp("<red>Material not found."));
      return false;
    }
    item.setMaterial(material);
    sender.sendMessage(comp("<green>Item material set."));
    updateItem(p, item);
    return true;
  }

  private static boolean modifyEnchantments(@NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> enchantments <add|remove> [enchantment] <level>");
      return false;
    }
    if (args[3].equalsIgnoreCase("add")) {
      if (args.length < 6) {
        p.sendMessage("Usage: /item modify <id> enchantments add <enchantment> <level>");
        return false;
      }
      int level = 1;
      try {
        level = Integer.parseInt(args[5]);
      } catch (NumberFormatException e) {
        p.sendMessage("Invalid level.");
        return false;
      }
      item.addEnchantment(Enchantment.getEnchantment(args[4]), level);
      p.sendMessage("Enchantments not working for now.");
      return true;
    }
    if (args[3].equalsIgnoreCase(REMOVE)) {
      if (args.length < 5) {
        p.sendMessage("Usage: /item modify <id> enchantments remove <enchantment>");
        return false;
      }
      item.removeEnchantment(Enchantment.getEnchantment(args[4]));
      p.sendMessage("Enchantments not working for now.");
    }
    updateItem(p, item);
    return true;
  }

  private static boolean modifyAttributes(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p, Item item) {
    if (args.length < 4) {
      p.sendMessage("Usage: /item modify <id> attributes <add|remove> [attribute] [value]");
      return false;
    }
    if (args[3].equalsIgnoreCase("add")) {
      if (args.length < 6) {
        p.sendMessage("Usage: /item modify <id> attributes add <attribute> <value>");
        return false;
      }
      Attribute attribute = null;
      try {
        attribute = Attribute.valueOf(args[4]);
      } catch (Exception e) {
        sender.sendMessage(comp("<red>Attribute not found."));
        error("Error on attribute affectation on item: " + args[1] + "\n" + e);
        return false;
      }
      double value;
      try {
        value = Double.parseDouble(args[5]);
      } catch (NumberFormatException e) {
        sender.sendMessage(comp("<red>Invalid value."));
        return false;
      }
      item.addAttribute(attribute, value);
      sender.sendMessage(comp("<green>Attribute added."));
      return false;
    }
    if (args[3].equalsIgnoreCase(REMOVE)) {
      if (args.length < 5) {
        p.sendMessage("Usage: /item modify <id> attributes remove <attribute>");
        return false;
      }
      Attribute attribute = null;
      try {
        attribute = Attribute.valueOf(args[4]);
      } catch (Exception e) {
        sender.sendMessage(comp("<red>Attribute not found."));
        error("Error on attribute affectation on item: " + args[1] + "\n" + e);
        return false;
      }
      item.removeAttribute(attribute);
      sender.sendMessage(comp("<green>Attribute removed."));
    }
    updateItem(p, item);
    return true;
  }

  private static void give(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p) {
    if (args.length < 2) {
      sender.sendMessage("Usage: /item give <id>");
      return;
    }

    ItemManager im = Manager.getInstance(ItemManager.class);
    if (!im.getItemIDs().contains(args[1])) {
      sender.sendMessage(comp(DOES_NOT_EXIST));
      return;
    }

    Item item = im.getItem(args[1]);

    if (args.length == 2) {
      p.getInventory().addItem(item.toItemStack());
      sender.sendMessage(comp("<green>Item given."));
      return;
    }

    Player target = Bukkit.getPlayer(args[2]);
    if (target == null) {
      sender.sendMessage(comp("<red>Player not found."));
      return;
    }

    target.getInventory().addItem(item.toItemStack());
    sender.sendMessage(comp("<green>Item given."));
  }

  private static void delete(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, Player p) {
    if (args.length < 2) {
      p.sendMessage("Usage: /item delete <id>");
      return;
    }
    ItemManager im = Manager.getInstance(ItemManager.class);
    if (!im.getItemIDs().contains(args[1])) {
      sender.sendMessage(comp(DOES_NOT_EXIST));
      return;
    }
    im.removeItem(args[1]);
    sender.sendMessage(comp("<green>Item deleted."));
  }

  private static void create(@NotNull CommandSender sender, @NotNull String @NotNull [] args) {
    if (args.length < 2) {
      sender.sendMessage("Usage: /item create <id>");
      return;
    }
    ItemManager im = Manager.getInstance(ItemManager.class);
    if (im.getItemIDs().contains(args[1])) {
      sender.sendMessage(comp("<red>An item with that ID already exists."));
      return;
    }
    Item item = new Item();
    item.setId(args[1]);
    im.addItem(item);
    sender.sendMessage(
        comp(
            "<green>Item created. You can now modify it using "
                + "<click:suggest_command:/item "
                + "modify "
                + args[1]
                + "> /item modify "
                + args[1]
                + "</click>"));
  }

  private static void list(@NotNull CommandSender sender, Player p) {
    ItemManager im = Manager.getInstance(ItemManager.class);
    if (im.getItemIDs().isEmpty()) {
      sender.sendMessage(comp("<red>There are no items."));
      return;
    }
    sender.sendMessage("Items:");
    im.getItemIDs().forEach(p::sendMessage);
  }

  @Nullable
  private static List<String> modify(@NotNull String @NotNull [] args) {
    if (args.length == 5 && (args[0].equalsIgnoreCase(MODIFY))) {
      if (args[2].equalsIgnoreCase(ENCHANTMENTS)) {
        return Arrays.stream(Enchantment.values())
            .map(
                name -> {
                  if (name.name().startsWith(args[3].toUpperCase())) return name.name();
                  return null;
                })
            .toList();
      } else if (args[2].equalsIgnoreCase(ATTRIBUTES)) {
        return Arrays.stream(Attribute.values())
            .map(
                name -> {
                  if (name.name().startsWith(args[3].toUpperCase())) return name.name();
                  return null;
                })
            .toList();
      }
    }
    return null;
  }

  private static void updateItem(@NotNull Player p, @NotNull Item newItem) {
    p.getInventory().setItemInMainHand(newItem.toItemStack());
  }

  @Nullable
  private static List<String> modifyCompletion(@NotNull String @NotNull [] args) {
    if (args.length == 4 && (args[0].equalsIgnoreCase(MODIFY))) {
      if (args[2].equalsIgnoreCase(RARITY)) {
        return Arrays.stream(Rarity.values()).map(Enum::name).toList();
      } else if (args[2].equalsIgnoreCase("type")) {
        return Arrays.stream(ItemType.values()).map(Enum::name).toList();
      } else if (args[2].equalsIgnoreCase(MATERIAL)) {
        return Arrays.stream(Material.values())
            .map(
                name -> {
                  if (name.name().startsWith(args[3].toUpperCase())) return name.name();
                  return null;
                })
            .toList();
      } else if (args[2].equalsIgnoreCase(ENCHANTMENTS)) {
        return List.of("add", REMOVE);
      } else if (args[2].equalsIgnoreCase(ATTRIBUTES)) {
        return List.of("add", REMOVE);
      } else if (args[2].equalsIgnoreCase(ABILITIES)) {
        return List.of("add", REMOVE);
      } else if (args[2].equalsIgnoreCase(CUSTOMMODELDATA)) {
        try {
          Integer.parseInt(args[3]);
        } catch (NumberFormatException e) {
          return List.of("1");
        }
      }
    }
    return null;
  }

  @Nullable
  private static List<String> argsLengthThree(@NotNull String @NotNull [] args) {
    if (args.length == 3) {
      if (args[0].equalsIgnoreCase(MODIFY)) {
        return List.of(
            "name",
            MATERIAL,
            ENCHANTMENTS,
            ATTRIBUTES,
            ABILITIES,
            CUSTOMMODELDATA,
            RARITY,
            "type",
            "description");
      } else if (args[0].equalsIgnoreCase("give")) {
        return Bukkit.getOnlinePlayers().stream().map(Player::getName).toList();
      }
    }
    return null;
  }

  /**
   * Executes the given command, returning its success. <br>
   * If false is returned, then the "usage" plugin.yml entry for this command (if defined) will be
   * sent to the player.
   *
   * @param sender Source of the command
   * @param command Command which was executed
   * @param label Alias of the command which was used
   * @param args Passed command arguments
   * @return true if a valid command, otherwise false
   */
  @Override
  public boolean onCommand(
      @NotNull CommandSender sender,
      @NotNull Command command,
      @NotNull String label,
      @NotNull String[] args) {
    if (!(sender instanceof Player p)) {
      sender.sendMessage("You must be a player to use this command.");
      return true;
    }

    if (!p.hasPermission("sr.admin.item")) {
      p.sendMessage("You do not have permission to use this command.");
      return true;
    }

    if (args.length == 0) {
      p.sendMessage("Usage: /item <list|create|delete|give|modify>");
      return true;
    }

    if (args[0].equalsIgnoreCase("list")) {
      list(sender, p);
      return true;
    }

    if (args[0].equalsIgnoreCase("create")) {
      create(sender, args);
      return true;
    }

    if (args[0].equalsIgnoreCase(DELETE)) {
      delete(sender, args, p);
      return true;
    }

    if (args[0].equalsIgnoreCase("give")) {
      give(sender, args, p);
      return true;
    }

    if (args[0].equalsIgnoreCase(MODIFY)) {
      modify(sender, args, p);
      return true;
    }

    return true;
  }

  /**
   * Requests a list of possible completions for a command argument.
   *
   * @param sender Source of the command. For players tab-completing a command inside of a command
   *     block, this will be the player, not the command block.
   * @param command Command which was executed
   * @param label Alias of the command which was used
   * @param args The arguments passed to the command, including final partial argument to be
   *     completed
   * @return A List of possible completions for the final argument, or null to default to the
   *     command executor
   */
  @Override
  public @Nullable List<String> onTabComplete(
      @NotNull CommandSender sender,
      @NotNull Command command,
      @NotNull String label,
      @NotNull String[] args) {
    if (!(sender instanceof Player) || !sender.hasPermission("sr.admin.item")) {
      return List.of("");
    }

    if (args.length == 1) {
      return List.of("list", "create", DELETE, "give", MODIFY);
    }

    if (args.length == 2
        && (args[0].equalsIgnoreCase(DELETE)
            || args[0].equalsIgnoreCase("give")
            || args[0].equalsIgnoreCase(MODIFY))) {
      ItemManager im = Manager.getInstance(ItemManager.class);
      return im.getItemIDs();
    }

    List<String> completions = argsLengthThree(args);
    if (completions != null) return completions;

    completions = modifyCompletion(args);
    if (completions != null) return completions;

    completions = modify(args);
    if (completions != null) return completions;

    if (args.length == 6
        && (args[0].equalsIgnoreCase(MODIFY))
        && (args[2].equalsIgnoreCase(ATTRIBUTES))) {
      try {
        Double.parseDouble(args[5]);
      } catch (NumberFormatException e) {
        return List.of("1.0");
      }
    }

    return List.of("");
  }
}
