package me.unurled.sacredrealms.sr.components.item;

import static me.unurled.sacredrealms.sr.utils.Component.textComp;
import static me.unurled.sacredrealms.sr.utils.Items.lore;
import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import java.lang.reflect.Type;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.item.abilities.Ability;
import me.unurled.sacredrealms.sr.components.item.abilities.AbilityDeserializer;
import me.unurled.sacredrealms.sr.components.item.enchantments.Enchantment;
import org.bukkit.Material;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;

/* <b><color of the rarity>rarity
 * <dark_gray>Description: blablabla
 *   <dark_gray>blabbla
 *   <dark_gray>blabla
 * <gray>Attributes:
 *   <gray>➤ Health: <green>+100
 *   <gray>➤ Strength: <red>+10
 *   <gray>➤ Mana: <dark_aqua>+10
 * <gray>Abilities:
 *   <gray>➤ Ability 1,<yellow> On Rightclick unleash a powerful attack
 *               <dark_gray>cooldown: 10s,
 *               <dark_gray>damage: 100,
 *               <dark_gray>mana cost: <dark_aqua>50
 *   <gray>➤ Ability 2,<yellow> On Rightclick unleash a powerful attack
 *               <dark_gray>cooldown: 10s,
 *               <dark_gray>damage: 100,
 *               <dark_gray>mana cost: <dark_aqua>50
 * <dark_purple>Enchantments:
 *   <blue>➤ Enchantment <if max level gold>1
 *   <blue>➤ Enchantment <if max level gold>2
 * */
public class Item {

  @NotNull private String id;

  @NotNull private String name;

  @NotNull private Material material;

  @NotNull private String description;

  @NotNull private Integer customModelData;

  @NotNull private Rarity rarity;

  @NotNull private ItemType type;

  @NotNull private Map<Attribute, Double> attributes;

  @NotNull private Map<Enchantment, Integer> enchantments;

  @NotNull private List<Ability> abilities;

  public Item() {
    this.id = "";
    this.name = "";
    this.material = Material.AIR;
    this.description = "";
    this.rarity = Rarity.COMMON;
    this.customModelData = 0;
    this.abilities = List.of();
    this.attributes = new EnumMap<>(Attribute.class);
    this.enchantments = new HashMap<>();
    this.type = ItemType.MISC;
  }

  public Item(@NotNull String id, @NotNull String name, @NotNull Material material) {
    this.id = id;
    this.name = name;
    this.material = material;
    this.description = "";
    this.rarity = Rarity.COMMON;
    this.customModelData = 0;
    this.abilities = List.of();
    this.attributes = new EnumMap<>(Attribute.class);
    this.enchantments = new HashMap<>();
    this.type = ItemType.MISC;
  }

  public @NotNull String getId() {
    return id;
  }

  public void setId(@NotNull String arg) {
    this.id = arg;
  }

  public @NotNull String getName() {
    return name;
  }

  public void setName(@NotNull String name) {
    this.name = name;
  }

  public @NotNull Material getMaterial() {
    return material;
  }

  public void setMaterial(@NotNull Material material) {
    this.material = material;
  }

  public @NotNull String getDescription() {
    return description;
  }

  public void setDescription(@NotNull String description) {
    this.description = description;
  }

  public @NotNull Rarity getRarity() {
    return rarity;
  }

  public void setRarity(@NotNull Rarity rarity) {
    this.rarity = rarity;
  }

  public @NotNull ItemType getType() {
    return type;
  }

  public void setType(@NotNull ItemType type) {
    this.type = type;
  }

  public @NotNull Map<Attribute, Double> getAttributes() {
    return attributes;
  }

  public void setAttributes(@NotNull Map<Attribute, Double> attributes) {
    this.attributes = attributes;
  }

  public Double getAttribute(Attribute attribute) {
    return attributes.get(attribute);
  }

  public void addAttribute(Attribute attribute, double value) {
    attributes.put(attribute, value);
  }

  public void removeAttribute(Attribute attribute) {
    attributes.remove(attribute);
  }

  public @NotNull Map<Enchantment, Integer> getEnchantments() {
    return enchantments;
  }

  @SuppressWarnings("unused")
  public void setEnchantments(@NotNull Map<Enchantment, Integer> enchantments) {
    this.enchantments = enchantments;
  }

  @SuppressWarnings("unused")
  public void addEnchantment(Enchantment enchantment, int level) {
    enchantments.put(enchantment, level);
  }

  @SuppressWarnings("unused")
  public void removeEnchantment(Enchantment enchantment) {
    enchantments.remove(enchantment);
  }

  public @NotNull List<Ability> getAbilities() {
    return abilities;
  }

  @SuppressWarnings("unused")
  public void setAbilities(@NotNull List<Ability> abilities) {
    this.abilities = abilities;
  }

  @SuppressWarnings("unused")
  public void addAbility(Ability ability) {
    abilities.add(ability);
  }

  @SuppressWarnings("unused")
  public void removeAbility(Ability ability) {
    abilities.remove(ability);
  }

  @SuppressWarnings("unused")
  public @NotNull Integer getCustomModelData() {
    return customModelData;
  }

  public void setCustomModelData(@NotNull Integer customModelData) {
    this.customModelData = customModelData;
  }

  public ItemStack toItemStack() {
    ItemStack item = new ItemStack(material);
    ItemMeta meta = item.getItemMeta();
    if (meta == null) {
      return null;
    }

    PersistentDataContainer p = meta.getPersistentDataContainer();
    p.set(ItemManager.ID, PersistentDataType.STRING, id);

    meta.displayName(textComp(name).color(rarity.getColor()));

    if (customModelData != 0) {
      meta.setCustomModelData(customModelData);
    }

    lore(item);

    item.setItemMeta(meta);
    return item;
  }

  @Override
  public String toString() {
    return "{"
        + "\"id\": \""
        + id
        + "\", \"name\": \""
        + name
        + "\", \"material\": \""
        + material
        + "\", \"description\": \""
        + description
        + "\", \"customModelData\": "
        + customModelData
        + "\", \"rarity\": \""
        + rarity
        + "\", \"type\": \""
        + type
        + "\", \"attributes\": \""
        + attributes
        + "\", \"enchantments\": \""
        + enchantments
        + "\", \"abilities\": \""
        + abilities
        + "\"}";
  }

  public void fromString(String item) {
    // extract every information from the string (it follows the same format as the toString method)
    Gson gson =
        new GsonBuilder().registerTypeAdapter(Ability.class, new AbilityDeserializer()).create();
    try {
      Map<String, Object> map =
          gson.fromJson(item, new TypeToken<Map<String, Object>>() {}.getType());

      this.id = (String) map.get("id");
      this.name = (String) map.get("name");
      this.material = Material.valueOf((String) map.get("material"));
      this.description = (String) map.get("description");
      this.customModelData = ((Double) map.get("customModelData")).intValue();
      this.rarity = Rarity.valueOf((String) map.get("rarity"));
      this.type = ItemType.valueOf((String) map.get("type"));

      Type attributeType = new TypeToken<HashMap<Attribute, Double>>() {}.getType();
      this.attributes = gson.fromJson((String) map.get("attributes"), attributeType);

      Type enchantmentType = new TypeToken<HashMap<Enchantment, Integer>>() {}.getType();
      this.enchantments = gson.fromJson((String) map.get("enchantments"), enchantmentType);

      Type abilityType = new TypeToken<List<Ability>>() {}.getType();
      this.abilities = gson.fromJson((String) map.get("abilities"), abilityType);
    } catch (Exception e) {
      error("Failed to parse item from string: " + item + "\n" + e.getMessage());
    }
  }
}
