package me.unurled.sacredrealms.sr.components.item;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import java.util.HashMap;
import java.util.List;
import me.unurled.sacredrealms.sr.SR;
import me.unurled.sacredrealms.sr.data.DataHandler;
import me.unurled.sacredrealms.sr.data.DataManager;
import me.unurled.sacredrealms.sr.managers.Manager;
import org.bukkit.NamespacedKey;
import org.bukkit.inventory.ItemStack;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;

public class ItemManager extends Manager {
  public static final NamespacedKey ID = new NamespacedKey("sr", "id");

  private final HashMap<String, Item> items;

  public ItemManager() {
    super();
    items = new HashMap<>();
  }

  /** Save the data */
  @Override
  public void saveData() {
    DataManager dm = Manager.getInstance(DataManager.class);
    DataHandler dh = dm.getDataHandler();

    Gson gson = new GsonBuilder().registerTypeAdapter(Item.class, new ItemSerializer()).create();

    items.forEach((id, item) -> dh.set("sr.items." + id, gson.toJson(item)));
  }

  /** Load the data */
  @Override
  public void loadData() {
    DataManager dm = Manager.getInstance(DataManager.class);
    DataHandler dh = dm.getDataHandler();

    if (dh == null) {
      SR.getInstance().getLogger().severe("DataHandler is null in ItemManager.loadData()!");
      return;
    }

    Gson gson = new GsonBuilder().registerTypeAdapter(Item.class, new ItemDeserializer()).create();

    dh.getKeysAll("sr.items")
        .forEach(
            key -> {
              Item item = gson.fromJson(dh.get(key), Item.class);
              items.put(item.getId(), item);
            });
  }

  public void saveItem(@NotNull Item item) {
    DataManager dm = Manager.getInstance(DataManager.class);
    DataHandler dh = dm.getDataHandler();

    Gson gson = new GsonBuilder().registerTypeAdapter(Item.class, new ItemSerializer()).create();

    dh.set("sr.items." + item.getId(), gson.toJson(item));
  }

  public void addItem(Item item) {
    items.put(item.getId(), item);
  }

  public Item getItem(String id) {
    return items.get(id);
  }

  public Item getItem(ItemStack item) {
    if (item == null) return null;
    if (item.getItemMeta() == null) return null;
    if (item.getItemMeta().getPersistentDataContainer().has(ID, PersistentDataType.STRING)) {
      return getItem(
          item.getItemMeta().getPersistentDataContainer().get(ID, PersistentDataType.STRING));
    }
    return null;
  }

  public void removeItem(String id) {
    items.remove(id);
  }

  public boolean isItem(String id) {
    return items.containsKey(id);
  }

  public List<String> getItemIDs() {
    return List.copyOf(items.keySet());
  }
}
